package com.dms.docx4j;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.List;

import org.docx4j.dml.wordprocessingDrawing.Inline;
import org.docx4j.model.structure.SectionWrapper;
import org.docx4j.openpackaging.Base;
import org.docx4j.openpackaging.contenttype.ContentTypeManager;
import org.docx4j.openpackaging.exceptions.Docx4JException;
import org.docx4j.openpackaging.exceptions.InvalidFormatException;
import org.docx4j.openpackaging.io.Load;
import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
import org.docx4j.openpackaging.parts.Part;
import org.docx4j.openpackaging.parts.WordprocessingML.BinaryPartAbstractImage;
import org.docx4j.openpackaging.parts.WordprocessingML.HeaderPart;
import org.docx4j.openpackaging.parts.WordprocessingML.MainDocumentPart;
import org.docx4j.relationships.Relationship;
import org.docx4j.wml.Hdr;
import org.docx4j.wml.HdrFtrRef;
import org.docx4j.wml.HeaderReference;
import org.docx4j.wml.ObjectFactory;
import org.docx4j.wml.SectPr;

import com.itextpdf.text.pdf.codec.Base64.InputStream;

public class CreateHeaderFromXML {

	private static ObjectFactory objectFactory = new ObjectFactory();

	public static void main(String[] args) throws Exception {

		String file = "/template.docx"; 
		String baseDir = System.getProperty("user.dir");
		String expandedDir = "/headers";
		
		replaceHeader(file, baseDir, expandedDir);
		
	
		
	}

	
	public static void replaceHeader(String file, String baseDir, String expandedDir) throws Exception {
        WordprocessingMLPackage wordMLPackage = null;
        if (file.endsWith(".docx")) {
            
                wordMLPackage = WordprocessingMLPackage.load(new File(baseDir + file));
        		MainDocumentPart mainDocumentPart = wordMLPackage.getMainDocumentPart();
  
                FileInputStream in = new FileInputStream(baseDir + expandedDir + "/[Content_Types].xml");
                ContentTypeManager externalCtm = new ContentTypeManager();

                externalCtm.parseContentTypesFile(in);

                // Example of a part which become a rel of the word document
                in = new FileInputStream(baseDir + expandedDir + "/word/header1.xml");
                attachForeignPart(wordMLPackage,
                        wordMLPackage.getMainDocumentPart(),
                        externalCtm,
                        "word/header1.xml",
                        in);

                // Example of a part which become a rel of the package
                in = new FileInputStream(baseDir + expandedDir + "/docProps/app.xml");
                attachForeignPart(wordMLPackage, wordMLPackage,
                        externalCtm, "docProps/app.xml", in);
                
                Relationship relationship = mainDocumentPart.getRelationshipsPart().getRelationshipByID("rId6");
                
            	createHeaderReference(wordMLPackage,relationship);
                
                wordMLPackage.save(new File(baseDir + "/header-added.docx"));
        }
    }


	public static void attachForeignPart(WordprocessingMLPackage wordMLPackage,Base attachmentPoint,ContentTypeManager foreignCtm,
			String resolvedPartUri, FileInputStream is) throws Exception {
		Part foreignPart = Load.getRawPart(is, foreignCtm, resolvedPartUri, null);
		// the null means this won't work for an AlternativeFormatInputPart
		attachmentPoint.addTargetPart(foreignPart);
		// Add content type
		ContentTypeManager packageCtm = wordMLPackage.getContentTypeManager();
		packageCtm.addOverrideContentType(foreignPart.getPartName().getURI(), foreignPart.getContentType());
	}

	public static void createHeaderReference(WordprocessingMLPackage wordprocessingMLPackage,Relationship relationship )
	        throws InvalidFormatException {
	
	     List<SectionWrapper> sections = wordprocessingMLPackage.getDocumentModel().getSections();
	        
	     SectPr sectPr = sections.get(sections.size() - 1).getSectPr();
	     
	     // There is always a section wrapper, but it might not contain a sectPr
	     if (sectPr==null ) {
	        sectPr = objectFactory.createSectPr();
	        wordprocessingMLPackage.getMainDocumentPart().addObject(sectPr);
	        sections.get(sections.size() - 1).setSectPr(sectPr);
	     }
	}
}
